﻿namespace MyCompany.Crt.Common
{
    using Microsoft.ApplicationInsights;
    using Microsoft.ApplicationInsights.DataContracts;
    using Microsoft.Dynamics.Commerce.Runtime;
    using Microsoft.Dynamics.Retail.Diagnostics;
    using System.Collections.Generic;

    public static class LoggerExtensions
    {
        public static TraceTelemetry CreateTraceWithSessionContext(this TelemetryClient logger, SeverityLevel level)
        {
            var trace = new TraceTelemetry()
            {
                SeverityLevel = level,
            };

            trace.Properties.AddRange(GetSessionContextProperties());
            return trace;
        }

        private static IEnumerable<KeyValuePair<string, string>> GetSessionContextProperties()
        {
            // We are getting the details from the RetailLogger, just so we can trace them with our custom logger. That way we can correlate custom traces (coming through this logger)
            // with oob traces (enabled via Operations Insights) correctly.  ActivityId, scale unit, Session Id, User id, etc.
            Dictionary<string, string> properties = new Dictionary<string, string>();
            string key;
            string val = null;

            key = "ActivityId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "AppSessionId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "AppWindowId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "Application";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "DeviceRecordId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "DiagnosticsVersion";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "EventId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }
            key = "EventVersion";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }
            key = "HighPrecisionEventOrder";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "ScaleUnitId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }
            key = "ScaleUnitName";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }
            key = "ScaleUnitRegion";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }
            key = "TenantId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "TerminalRecordId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "UserSessionId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "applicationIntent";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "callWasSuccessful";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            key = "correlationId";
            val = GetSessionPropertyValue(key);
            if (val != null)
            {
                properties.Add(key, val);
            }

            return properties;
        }

        private static string GetSessionPropertyValue(string name)
        {
            if (RetailLogger.Log != null)
            {
                object val = null;
                val = RetailLogger.Log.GetSessionPropertyValue(name);
                if (val != null)
                {
                    return val.ToString();
                }
            }

            return null;
        }
    }
}
