﻿namespace MyCompany.Crt.RequestHandlers
{
    using Microsoft.Dynamics.Commerce.Runtime;
    using Microsoft.Dynamics.Commerce.Runtime.Messages;
    using Microsoft.Dynamics.Commerce.Runtime.RealtimeServices.Messages;
    using MyCompany.Crt.Common;
    using MyCompany.Crt.Managers;
    using System;
    using System.Threading.Tasks;

    public class GetUserDefinedSecretStringValueRealtimeRequestHandlerWithCaching : SingleAsyncRequestHandler<GetUserDefinedSecretStringValueRealtimeRequest>
    {
        protected override Task<Response> Process(GetUserDefinedSecretStringValueRealtimeRequest request)
        {
            ThrowIf.Null(request, "request");
            var telemetryLogger = Logger.GetLogger(request.RequestContext);
            var cacheAccessor = GetSecretStringValueRealtimeCacheAccessor();
            return Task.FromResult(cacheAccessor.GetOrCreate(request.SecretName, v =>
            {
                return FetchSecretString(request).Result;
            }, absoluteExpirationRelativeToNow: TimeSpan.FromMinutes(60), slidingExpiration: null, telemetryLogger));
        }

        private async Task<Response> FetchSecretString(GetUserDefinedSecretStringValueRealtimeRequest request)
        {
            return await request.RequestContext.Runtime.ExecuteNextAsync<GetUserDefinedSecretStringValueRealtimeResponse>(this, request, request.RequestContext, false).ConfigureAwait(false);
        }
        private IResponseCacheAccessor GetSecretStringValueRealtimeCacheAccessor()
        {
            return new SecretStringValueRealtimeCacheAccessor();
        }
    }
}